(*******************************************************************************
**
**  TIconLabel 1.00
**  copyright  LogicX websolutions company
**
**  Programming by: Markus Thielen [mt@logicx.de]
**
**  Comments/questions/bugs? Please write me...
**
**  Released as copyrighted freeware.
**
**  Disclaimer:
**  LOGICX SOFTWARE TAKES NO RESPONSIBILITY FOR ANYTHING
**  HARMED OR DAMAGED BY THIS PIECE OF CODE. IF YOU MODIFY
**  IT, THE AUTHOR KEEPS THE RIGHT TO BE SENT A COPY OF
**  THE MODIFIED SOURCE. YOU CAN FREELY MODIFY AND REDISTRIBUTE
**  FOR NON-PROFIT AS LONG AS THE AUTHOR IS MENTIONED.
**  ALL OTHER RIGHTS RESERVED.
**
**  If you add some interesting things to the component please write an email!
**
**  Visit our Homepage at www.logicx.de
**
*******************************************************************************)
unit IconLabel;

interface

uses Messages, Windows, SysUtils, Classes, Controls, Forms, Menus, Graphics,
  StdCtrls;

type
  TIconLabel = class(TCustomLabel)
  private
    FPicture: TPicture;
    FWordWrap: Boolean;
    FAlignment: TAlignment;
    FLayout: TTextLayout;
    procedure SetPicture(Value: TPicture);
    procedure SetWordWrap(Value: Boolean);
    procedure SetLayout(Value: TTextLayout);
    procedure SetAlignment(Value: TAlignment);
    { Private-Deklarationen }
  protected
    procedure Paint; override;
    { Protected-Deklarationen }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destory(AOwner: TComponent);
    { Public-Deklarationen }
  published
    { Published-Deklarationen }
    property Icon:TPicture read FPicture write SetPicture;
    property Align;
    property Alignment;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TIconLabel]);
end;

constructor TIconLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetAutoSize(False);
  ControlStyle := ControlStyle - [csOpaque];
  FPicture := TPicture.Create;
end;

Destructor TIconLabel.Destory(AOwner: TComponent);
begin
  inherited;
  FPicture.Free;
end;

procedure TIconLabel.SetWordWrap(Value: Boolean);
begin
  inherited;
  FWordWrap := Value;
end;

procedure TIconLabel.SetLayout(Value: TTextLayout);
begin
  inherited;
  FLayout := Value;
end;

procedure TIconLabel.SetAlignment(Value: TAlignment);
begin
  inherited;
  FAlignment := Value;
end;

procedure TIconLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Longint;
begin
  with Canvas do
  begin
    //Icon reinhauen
    if icon.Graphic <> nil then
    begin
      Icon.Graphic.Transparent := True;
      Draw(0,0, Icon.Graphic);
    end;
    //Text
    Brush.Style := bsClear;
    Rect := ClientRect;
    rect.Left := rect.left+icon.Width+5;
    
    { DoDrawText takes care of BiDi alignments }
    DrawStyle := DT_EXPANDTABS or WordWraps[FWordWrap] or Alignments[FAlignment];
    { Calculate vertical layout }
    if FLayout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if FLayout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    DoDrawText(Rect, DrawStyle);
  end;
end;

procedure TIconLabel.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
  width := width+value.Width+5;
end;

end.
